@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2007 OpenedHand Ltd.
+ * Copyright (C) 2006, 2007, 2008 OpenedHand Ltd.
  *
  * Author: Jorn Baayen <jorn@openedhand.com>
  *
@@ -35,8 +35,8 @@
 
 #include "gupnp-control-point.h"
 #include "gupnp-context-private.h"
-#include "gupnp-device-proxy-private.h"
-#include "gupnp-service-proxy-private.h"
+#include "gupnp-resource-factory-private.h"
+#include "http-headers.h"
 #include "xml-util.h"
 
 G_DEFINE_TYPE (GUPnPControlPoint,
@@ -44,6 +44,8 @@ G_DEFINE_TYPE (GUPnPControlPoint,
                GSSDP_TYPE_RESOURCE_BROWSER);
 
 struct _GUPnPControlPointPrivate {
+        GUPnPResourceFactory *factory;
+
         GList *devices;
         GList *services;
 
@@ -52,6 +54,11 @@ struct _GUPnPControlPointPrivate {
         GList *pending_gets;
 };
 
+enum {
+        PROP_0,
+        PROP_RESOURCE_FACTORY,
+};
+
 enum {
         DEVICE_PROXY_AVAILABLE,
         DEVICE_PROXY_UNAVAILABLE,
@@ -140,6 +147,11 @@ gupnp_control_point_dispose (GObject *object)
 
         control_point = GUPNP_CONTROL_POINT (object);
 
+        if (control_point->priv->factory) {
+                g_object_unref (control_point->priv->factory);
+                control_point->priv->factory = NULL;
+        }
+
         while (control_point->priv->devices) {
                 g_object_unref (control_point->priv->devices->data);
                 control_point->priv->devices =
@@ -163,10 +175,11 @@ gupnp_control_point_dispose (GObject *object)
                 data = control_point->priv->pending_gets->data;
 
                 context = gupnp_control_point_get_context (control_point);
-                session = _gupnp_context_get_session (context);
+                session = gupnp_context_get_session (context);
 
-                soup_message_set_status (data->message, SOUP_STATUS_CANCELLED);
-                soup_session_cancel_message (session, data->message);
+                soup_session_cancel_message (session,
+                                             data->message,
+                                             SOUP_STATUS_CANCELLED);
 
                 get_description_url_data_free (data);
         }
@@ -204,7 +217,7 @@ process_service_list (xmlNode           *element,
                       const char        *udn,
                       const char        *service_type,
                       const char        *description_url,
-                      SoupUri           *url_base)
+                      SoupURI           *url_base)
 {
         for (element = element->children; element; element = element->next) {
                 xmlChar *prop;
@@ -234,7 +247,9 @@ process_service_list (xmlNode           *element,
                 context = gupnp_control_point_get_context (control_point);
 
                 /* Create proxy */
-                proxy = _gupnp_service_proxy_new (context,
+                proxy = gupnp_resource_factory_create_service_proxy
+                                                 (control_point->priv->factory,
+                                                  context,
                                                   doc,
                                                   element,
                                                   udn,
@@ -261,7 +276,7 @@ process_device_list (xmlNode           *element,
                      const char        *udn,
                      const char        *service_type,
                      const char        *description_url,
-                     SoupUri           *url_base)
+                     SoupURI           *url_base)
 {
         for (element = element->children; element; element = element->next) {
                 xmlNode *children;
@@ -323,12 +338,14 @@ process_device_list (xmlNode           *element,
                         /* Create device proxy */
                         GUPnPDeviceProxy *proxy;
 
-                        proxy = _gupnp_device_proxy_new (context,
-                                                         doc,
-                                                         element,
-                                                         udn,
-                                                         description_url,
-                                                         url_base);
+                        proxy = gupnp_resource_factory_create_device_proxy
+                                        (control_point->priv->factory,
+                                         context,
+                                         doc,
+                                         element,
+                                         udn,
+                                         description_url,
+                                         url_base);
 
                         control_point->priv->devices =
                                 g_list_prepend
@@ -343,9 +360,9 @@ process_device_list (xmlNode           *element,
         }
 }
 
-/**
+/*
  * Called when the description document is loaded.
- **/
+ */
 static void
 description_loaded (GUPnPControlPoint *control_point,
                     XmlDocWrapper     *doc,
@@ -354,7 +371,7 @@ description_loaded (GUPnPControlPoint *control_point,
                     const char        *description_url)
 {
         xmlNode *element;
-        SoupUri *url_base;
+        SoupURI *url_base;
 
         /* Save the URL base, if any */
         element = xml_util_get_element ((xmlNode *) doc->doc,
@@ -380,11 +397,12 @@ description_loaded (GUPnPControlPoint *control_point,
         soup_uri_free (url_base);
 }
 
-/**
+/*
  * Description URL downloaded.
- **/
+ */
 static void
-got_description_url (SoupMessage           *msg,
+got_description_url (SoupSession           *session,
+                     SoupMessage           *msg,
                      GetDescriptionURLData *data)
 {
         XmlDocWrapper *doc;
@@ -414,8 +432,8 @@ got_description_url (SoupMessage           *msg,
                 xmlDoc *xml_doc;
 
                 /* Parse response */
-                xml_doc = xmlParseMemory (msg->response.body,
-                                          msg->response.length);
+                xml_doc = xmlParseMemory (msg->response_body->data,
+                                          msg->response_body->length);
                 if (xml_doc) {
                         doc = xml_doc_wrapper_new (xml_doc);
 
@@ -448,14 +466,14 @@ got_description_url (SoupMessage           *msg,
         get_description_url_data_free (data);
 }
 
-/**
+/*
  * Downloads and parses (or takes from cache) @description_url,
  * creating:
  *  - A #GUPnPDeviceProxy for the device specified by @udn if @service_type
- *    is NULL.
+ *    is %NULL.
  *  - A #GUPnPServiceProxy for the service of type @service_type from the device
- *    specified by @udn if @service_type is not NULL.
- **/
+ *    specified by @udn if @service_type is not %NULL.
+ */
 static void
 load_description (GUPnPControlPoint *control_point,
                   const char        *description_url,
@@ -481,7 +499,7 @@ load_description (GUPnPControlPoint *control_point,
 
                 context = gupnp_control_point_get_context (control_point);
 
-                session = _gupnp_context_get_session (context);
+                session = gupnp_context_get_session (context);
 
                 data = g_slice_new (GetDescriptionURLData);
 
@@ -496,6 +514,9 @@ load_description (GUPnPControlPoint *control_point,
                         return;
                 }
 
+                http_request_set_user_agent (data->message);
+                http_request_set_accept_language (data->message);
+
                 data->control_point   = control_point;
 
                 data->udn             = g_strdup (udn);
@@ -508,7 +529,7 @@ load_description (GUPnPControlPoint *control_point,
 
 	        soup_session_queue_message (session,
                                             data->message,
-                                            (SoupMessageCallbackFn)
+                                            (SoupSessionCallback)
                                                    got_description_url,
                                             data);
         }
@@ -712,6 +733,47 @@ gupnp_control_point_resource_unavailable
         g_free (service_type);
 }
 
+static void
+gupnp_control_point_set_property (GObject      *object,
+                                  guint         property_id,
+                                  const GValue *value,
+                                  GParamSpec   *pspec)
+{
+        GUPnPControlPoint *control_point;
+
+        control_point = GUPNP_CONTROL_POINT (object);
+
+        switch (property_id) {
+        case PROP_RESOURCE_FACTORY:
+                control_point->priv->factory = 
+			GUPNP_RESOURCE_FACTORY (g_value_dup_object (value));
+                break;
+        default:
+                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+                break;
+        }
+}
+
+static void
+gupnp_control_point_get_property (GObject    *object,
+                                  guint       property_id,
+                                  GValue     *value,
+                                  GParamSpec *pspec)
+{
+        GUPnPControlPoint *control_point;
+
+        control_point = GUPNP_CONTROL_POINT (object);
+
+        switch (property_id) {
+        case PROP_RESOURCE_FACTORY:
+                g_value_set_object (value, control_point->priv->factory);
+                break;
+        default:
+                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+                break;
+        }
+}
+
 static void
 gupnp_control_point_class_init (GUPnPControlPointClass *klass)
 {
@@ -720,8 +782,10 @@ gupnp_control_point_class_init (GUPnPControlPointClass *klass)
 
         object_class = G_OBJECT_CLASS (klass);
 
-        object_class->dispose  = gupnp_control_point_dispose;
-        object_class->finalize = gupnp_control_point_finalize;
+        object_class->set_property = gupnp_control_point_set_property;
+        object_class->get_property = gupnp_control_point_get_property;
+        object_class->dispose      = gupnp_control_point_dispose;
+        object_class->finalize     = gupnp_control_point_finalize;
 
         browser_class = GSSDP_RESOURCE_BROWSER_CLASS (klass);
 
@@ -732,6 +796,24 @@ gupnp_control_point_class_init (GUPnPControlPointClass *klass)
 
         g_type_class_add_private (klass, sizeof (GUPnPControlPointPrivate));
 
+        /**
+         * GUPnPControlPoint:resource-factory
+         *
+         * The resource factory to use. Set to NULL for default factory.
+         **/
+        g_object_class_install_property
+                (object_class,
+                 PROP_RESOURCE_FACTORY,
+                 g_param_spec_object ("resource-factory",
+                                      "Resource Factory",
+                                      "The resource factory to use",
+                                      GUPNP_TYPE_RESOURCE_FACTORY,
+                                      G_PARAM_CONSTRUCT_ONLY |
+                                      G_PARAM_READWRITE |
+                                      G_PARAM_STATIC_NAME |
+                                      G_PARAM_STATIC_NICK |
+                                      G_PARAM_STATIC_BLURB));
+
         /**
          * GUPnPControlPoint::device-proxy-available
          * @control_point: The #GUPnPControlPoint that received the signal
@@ -822,17 +904,55 @@ gupnp_control_point_class_init (GUPnPControlPointClass *klass)
  * @context: A #GUPnPContext
  * @target: The search target
  *
+ * Create a new #GUPnPControlPoint with the specified @context and @target.
+ *
+ * @target should be a service or device name, such as
+ * <literal>urn:schemas-upnp-org:service:WANIPConnection:1</literal> or
+ * <literal>urn:schemas-upnp-org:device:MediaRenderer:1</literal>.
+ *
  * Return value: A new #GUPnPControlPoint object.
  **/
 GUPnPControlPoint *
 gupnp_control_point_new (GUPnPContext *context,
                          const char   *target)
+{
+        GUPnPResourceFactory *factory;
+
+        factory = gupnp_resource_factory_get_default ();
+
+        return gupnp_control_point_new_full (context,
+                                             factory,
+                                             target);
+}
+
+/**
+ * gupnp_control_point_new_full
+ * @context: A #GUPnPContext
+ * @factory: A #GUPnPResourceFactory
+ * @target: The search target
+ *
+ * Create a new #GUPnPControlPoint with the specified @context, @factory and
+ * @target.
+ *
+ * @target should be a service or device name, such as
+ * <literal>urn:schemas-upnp-org:service:WANIPConnection:1</literal> or
+ * <literal>urn:schemas-upnp-org:device:MediaRenderer:1</literal>.
+ *
+ * Return value: A new #GUPnPControlPoint object.
+ **/
+GUPnPControlPoint *
+gupnp_control_point_new_full (GUPnPContext         *context,
+                              GUPnPResourceFactory *factory,
+                              const char           *target)
 {
         g_return_val_if_fail (GUPNP_IS_CONTEXT (context), NULL);
+        g_return_val_if_fail (GUPNP_IS_RESOURCE_FACTORY (factory), NULL);
+        g_return_val_if_fail (target, NULL);
 
         return g_object_new (GUPNP_TYPE_CONTROL_POINT,
                              "client", context,
                              "target", target,
+                             "resource-factory", factory,
                              NULL);
 }
 
@@ -840,7 +960,9 @@ gupnp_control_point_new (GUPnPContext *context,
  * gupnp_control_point_get_context
  * @control_point: A #GUPnPControlPoint
  *
- * Return value: The #GUPnPContext associated with @control_point.
+ * Get the #GUPnPControlPoint associated with @control_point.
+ *
+ * Return value: The #GUPnPContext.
  **/
 GUPnPContext *
 gupnp_control_point_get_context (GUPnPControlPoint *control_point)
@@ -859,8 +981,10 @@ gupnp_control_point_get_context (GUPnPControlPoint *control_point)
  * gupnp_control_point_list_device_proxies
  * @control_point: A #GUPnPControlPoint
  *
- * Return value: A #GList of discovered #GUPnPDeviceProxy objects. Do not
- * free the list nor its elements.
+ * Get the #GList of discovered #GUPnPDeviceProxy objects. Do not free the list
+ * nor its elements.
+ *
+ * Return value: a #GList of #GUPnPDeviceProxy objects.
  **/
 const GList *
 gupnp_control_point_list_device_proxies (GUPnPControlPoint *control_point)
@@ -874,8 +998,10 @@ gupnp_control_point_list_device_proxies (GUPnPControlPoint *control_point)
  * gupnp_control_point_list_service_proxies
  * @control_point: A #GUPnPControlPoint
  *
- * Return value: A #GList of discovered #GUPnPServiceProxy objects. Do not
- * free the list nor its elements.
+ * Get the #GList of discovered #GUPnPServiceProxy objects. Do not free the list
+ * nor its elements.
+ *
+ * Return value: a #GList of #GUPnPServiceProxy objects.
  **/
 const GList *
 gupnp_control_point_list_service_proxies (GUPnPControlPoint *control_point)
@@ -884,3 +1010,19 @@ gupnp_control_point_list_service_proxies (GUPnPControlPoint *control_point)
 
         return (const GList *) control_point->priv->services;
 }
+
+/**
+ * gupnp_control_point_get_resource_factory
+ * @control_point: A #GUPnPControlPoint
+ *
+ * Get the #GUPnPResourceFactory used by the @control_point.
+ *
+ * Return value: A #GUPnPResourceFactory.
+ **/
+GUPnPResourceFactory *
+gupnp_control_point_get_resource_factory (GUPnPControlPoint *control_point)
+{
+        g_return_val_if_fail (GUPNP_IS_CONTROL_POINT (control_point), NULL);
+
+        return control_point->priv->factory;
+}
